/*
(c) 2000 David Laurie

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

gl_vfog.c

simple volumetric fog, courtesy of KrimZon aka 'scary' dave

*/

#include "quakedef.h"


/*
==========================
CalcVFog

per vertex traced fog blending for tomaz
==========================
*/
void CalcVFog (vec3_t point)
{
	trace_t t;
	float bigdist;
	float smalldist;
	float alpha;
	vec3_t fogline;

	VectorSubtract(point, r_refdef.vieworg, fogline);
	bigdist = Length(fogline);

	SV_RecursiveHullCheck(cl.worldmodel->hulls, 0, 1, 0, r_refdef.vieworg, point, &t);

	smalldist = bigdist * t.fraction;

	alpha = smalldist / 1000;
	glColor4f(0.3f,0.3f,0.3f,alpha);
}


//===============================================================
//===============================================================

/*
================
R_BlendFogSurface

Basically drawsequentialpoly but blends a colour
over the top. rgba style
================
*/
void R_BlendFogSurface (msurface_t *s)
{
	glpoly_t	*p;
	float		*v;
	int			i;
	vec3_t		rng;//dave

	if (s->flags && SURF_INFOG) 
	{
	}
	else
	{
		return;
	}

	//
	// draw an extra poly over the top of the surface
	// for doing global fog
	//

	if (gl_fogenable.value) glDisable(GL_FOG);

	//glDepthMask (0); 
	//GL_DisableMultitexture();
	glDisable (GL_ALPHA_TEST);
	glDisable (GL_TEXTURE_2D);
	//glDisable (GL_TEXTURE_1D);
	glShadeModel (GL_SMOOTH);
	glEnable (GL_BLEND);
	glBlendFunc (GL_ONE, GL_ONE_MINUS_SRC_ALPHA);

	p = s->polys;

	glBegin(GL_POLYGON);
	v = p->verts[0];
	for (i=0 ; i<p->numverts ; i++, v+= VERTEXSIZE)
	{
		// here calculates the volumetric fog from
		VectorCopy(v, rng);
		CalcVFog(rng);
		// a series of axis aligned boxes - adding to an rgb and alpha.
		glVertex3fv (v);
	}
	glEnd ();
	 
	glColor4f (1,1,1,1); 
	glDisable (GL_BLEND); 
	glEnable (GL_TEXTURE_2D); 
	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); 
	glEnable(GL_TEXTURE_2D);
	//glDepthMask (1);

	if (gl_fogenable.value) glEnable(GL_FOG);
}